@@ -1,6 +1,7 @@
 /*-
- * Copyright (C)2002..2018 @BABOLO http://www.babolo.ru/
- * Copyright (C) 2005 Michael Deryugin <dem@pikenet.ru>
+ * Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2005 Michael Deryugin <dem@pikenet.ru>
+ * pkg = babolo_pglib
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,12 +26,9 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO, Michael Deryugin\n\
-@(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: pglib_ip.c,v 1.10 2018/08/10 17:18:51 babolo Exp $";
-#endif /* not lint */
+#ident "@(#) Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) Copyright (C)2005 Michael Deryugin <dem@pikenet.ru>"
+#ident "@(#) $Id: pglib_ip.c,v 1.17 2025/11/12 02:39:09 babolo Exp $"
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -44,19 +42,78 @@ static const char rcsid[] = "$Id: pglib_ip.c,v 1.10 2018/08/10 17:18:51 babolo E
 #include <stdio.h>
 #include <fmgr.h>
 #include <funcapi.h>
+#include <postgres.h>
 #include <utils/inet.h>
-#include <access/tuptoaster.h>
 
 #define ALLOC(ptr) palloc((ptr))
 #define FREE(ptr)  pfree((ptr))
+#ifndef PG_GETARG_VARLENA_P
+# define PG_GETARG_VARLENA_P(A) do{heap_tuple_untoast_attr((varattrib*)PG_GETARG_POINTER(A))}while(0)
+#endif
 
-enum qchars {Q, T, F, SP};
+/* Error */
+#define ESYM (-1)
+
+/* Actions of qwalk */
+#define  SEARCH_T    1
+#define  SEARCH_F    2
+#define  SEARCH_ADDR 3
+#define  SET_BLANK   4
+#define  AGGREGATE   5
+
+#ifdef PG_MODULE_MAGIC
+    PG_MODULE_MAGIC;
+#endif
+
+#ifdef BLIN_OSLinux
+#   define NOTYPE 1
+#endif
+
+#if (PGSQL_VER > 8)
+#   define NOTYPE 1
+#   define INET_MEMSIZE(a)      (VARHDRSZ + offsetof(inet_struct, ipaddr) + ip_addrsize(a))
+#elif ((PGSQL_VER == 8) && (PGSQL_MOD > 1))
+#   define NOTYPE 1
+#   define ip_family(inetptr)   (((inet_struct*)VARDATA_ANY(inetptr))->family)
+#   define ip_bits(inetptr)     (((inet_struct*)VARDATA_ANY(inetptr))->bits)
+#   define ip_addr(inetptr)     (((inet_struct*)VARDATA_ANY(inetptr))->ipaddr)
+#   define ip_maxbits(inetptr)  (ip_family(inetptr) == PGSQL_AF_INET ? 32 : 128)
+#   define INET_MEMSIZE(a)      (VARHDRSZ + offsetof(inet_struct, ipaddr) + ip_addrsize(a))
+#else
+#   define ip_bits(inetptr)     (((inet_struct*)VARDATA(inetptr))->bits)
+#   define ip_family(inetptr)   (((inet_struct*)VARDATA(inetptr))->family)
+# ifndef NOTYPE
+#   define ip_type(inetptr)     (((inet_struct*)VARDATA(inetptr))->type)
+# endif
+#   define ip_maxbits(inetptr)  (ip_family(inetptr) == PGSQL_AF_INET ? 32 : 128)
+#   define INET_MEMSIZE(a)      (VARHDRSZ + ((char*)ip_addr(a) - (char*)VARDATA(a)) + ip_addrsize(a))
+
+# if PGSQL_VER > 7  /*  PG8 ip_addr  ipaddr */
+#   define ip_addr(inetptr)     ((inet_struct*)VARDATA(inetptr))->ipaddr
+# else
+#   define ip_addr(inetptr)     ((inet_struct*)VARDATA(inetptr))->ip_addr
+# endif
+#endif
+
+#if (PGSQL_VER > 8) || ((PGSQL_VER == 8) && (PGSQL_MOD > 2))
+#   define varattrib            struct varlena
+#endif
+
+#define fctx                    ((results*)funcctx->user_fctx)
+
+typedef struct {
+    u_int32_t current;
+    u_int32_t brodcast;
+    int32_t   masklen;
+} results;
 
 typedef struct {
     int32_t length;
     char data[1];
 } ipaggr;
 
+enum qchars {Q, T, F, SP};
+
 struct qstring {
     int32_t position;
     int32_t qlength;
@@ -76,56 +133,6 @@ struct qstate {
     int32_t stack[STACK_SIZE];
 };
 
-/* Error */
-#define ESYM (-1)
-
-/* Actions of qwalk */
-#define  SEARCH_T    1
-#define  SEARCH_F    2
-#define  SEARCH_ADDR 3
-#define  SET_BLANK   4
-#define  AGGREGATE   5
-
-#ifdef PG_MODULE_MAGIC
-    PG_MODULE_MAGIC;
-#endif
-
-#if (PGSQL_VER > 8)
-#   define NOTYPE 1
-#   define INET_MEMSIZE(a)      (VARHDRSZ + offsetof(inet_struct, ipaddr) + ip_addrsize(a))
-#elif ((PGSQL_VER == 8) && (PGSQL_MOD > 1))
-#   define NOTYPE 1
-#   define ip_family(inetptr)   (((inet_struct *)VARDATA_ANY(inetptr))->family)
-#   define ip_bits(inetptr)     (((inet_struct *)VARDATA_ANY(inetptr))->bits)
-#   define ip_addr(inetptr)     (((inet_struct *)VARDATA_ANY(inetptr))->ipaddr)
-#   define ip_maxbits(inetptr)  (ip_family(inetptr) == PGSQL_AF_INET ? 32 : 128)
-#   define INET_MEMSIZE(a)      (VARHDRSZ + offsetof(inet_struct, ipaddr) + ip_addrsize(a))
-#else
-#   define ip_bits(inetptr)     (((inet_struct *)VARDATA(inetptr))->bits)
-#   define ip_family(inetptr)   (((inet_struct *)VARDATA(inetptr))->family)
-#   define ip_type(inetptr)     (((inet_struct *)VARDATA(inetptr))->type)
-#   define ip_maxbits(inetptr)  (ip_family(inetptr) == PGSQL_AF_INET ? 32 : 128)
-#   define INET_MEMSIZE(a)      (VARHDRSZ + ((char*)ip_addr(a) - (char*)VARDATA(a)) + ip_addrsize(a))
-
-#   if PGSQL_VER > 7  /*  PG8 ip_addr  ipaddr */
-#       define ip_addr(inetptr) ((inet_struct *)VARDATA(inetptr))->ipaddr
-#   else
-#       define ip_addr(inetptr) ((inet_struct *)VARDATA(inetptr))->ip_addr
-#   endif
-#endif
-
-#if (PGSQL_VER > 8) || ((PGSQL_VER == 8) && (PGSQL_MOD > 2))
-#   define varattrib struct varlena
-#endif
-
-#define fctx                ((results*)funcctx->user_fctx)
-
-typedef struct {
-    u_int32_t current;
-    u_int32_t brodcast;
-    int32_t   masklen;
-} results;
-
 #if (PGSQL_VER < 9)
 static int
 ip_addrsize(inet *inetptr) {
@@ -136,11 +143,11 @@ ip_addrsize(inet *inetptr) {
 }   }
 #endif
 
-u_int32_t
+static u_int32_t
 ip_v4addr(inet *inetptr) {
-    int i;
+    int       i;
     u_int32_t a;
-    u_char *p;
+    u_char   *p;
 
     for (p = ip_addr(inetptr), a = 0, i = 0; i < ip_addrsize(inetptr); i++) {
         a = (a << 8) | p[i];
@@ -148,9 +155,9 @@ ip_v4addr(inet *inetptr) {
     return(a);
 }
 
-void
+static void
 toip_v4addr(inet *inetptr, u_int32_t a) {
-    int i;
+    int     i;
     u_char *p;
 
     for (p = ip_addr(inetptr), i = ip_addrsize(inetptr) - 1; i >= 0; i--) {
@@ -161,36 +168,44 @@ toip_v4addr(inet *inetptr, u_int32_t a) {
 PG_FUNCTION_INFO_V1(ip_set);
 Datum
 ip_set(PG_FUNCTION_ARGS) {
-    u_int32_t mask;
-    inet *src;
-    FuncCallContext  *funcctx;
-    MemoryContext oldcontext;
+    u_int32_t        mask;
+    inet            *src;
+    FuncCallContext *funcctx;
+    MemoryContext    oldcontext;
 
     if  (SRF_IS_FIRSTCALL()) {
         funcctx = SRF_FIRSTCALL_INIT();
         oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
         if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) { SRF_RETURN_DONE(funcctx); }
         src = PG_GETARG_INET_P(0);
-        if  (ip_family(src) == PGSQL_AF_INET6)
-            ereport(ERROR, ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
-                           , errmsg("ip_set: address family AF_INET6 not supported yet")
-                   )       );
-        if  (ip_family(src) != PGSQL_AF_INET)
-                                    elog(ERROR, "ip_set: unknown address family (%d)", ip_family(src));
+        if  (ip_family(src) == PGSQL_AF_INET6) {
+            ereport( ERROR
+                   , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
+                     , errmsg("ip_set: address family AF_INET6 not supported yet")
+                   ) );
+        }
+        if  (ip_family(src) != PGSQL_AF_INET) {
+            elog(ERROR, "ip_set: unknown address family (%d)", ip_family(src));
+        }
         funcctx->user_fctx = (results*)palloc(sizeof(results));
-        if  (fctx == NULL)
-                     ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
+        if  (!fctx) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
+        }
         MemSet(fctx, 0, sizeof(results));
         fctx->masklen = PG_GETARG_INT32(1);
-        if  (fctx->masklen > ip_maxbits(src) || fctx->masklen < 0)
-            ereport(ERROR, ( errcode(ERRCODE_INVALID_PARAMETER_VALUE)
-                           , errmsg("ip_set: Out of range")
-                           , errdetail( "Parameter $2 = %d out of range (0..%d)"
-                                      , fctx->masklen, ip_maxbits(src)
-                   )       )          );
-        if  (ip_bits(src) > fctx->masklen) { SRF_RETURN_DONE(funcctx); }
+        if  (fctx->masklen > ip_maxbits(src) || fctx->masklen < 0) {
+            ereport( ERROR
+                   , ( errcode(ERRCODE_INVALID_PARAMETER_VALUE)
+                     , errmsg("ip_set: Out of range")
+                     , errdetail( "Parameter $2 = %d out of range (0..%d)"
+                                , fctx->masklen, ip_maxbits(src)
+                   ) )          );
+        }
+        if  (ip_bits(src) > fctx->masklen) {
+            SRF_RETURN_DONE(funcctx);
+        }
         mask = 0xFFFFFFFF << (ip_maxbits(src) - ip_bits(src));
-        funcctx->max_calls = 1 << (fctx->masklen - ip_bits(src));     /* BEWARE!!! Overflow possible */
+        funcctx->max_calls = 1 << (fctx->masklen - ip_bits(src));          /* XXXX Overflow possible */
         fctx->current = mask & ip_v4addr(src);
         fctx->brodcast = ~mask | ip_v4addr(src);
         MemoryContextSwitchTo(oldcontext);
@@ -198,8 +213,9 @@ ip_set(PG_FUNCTION_ARGS) {
     funcctx = SRF_PERCALL_SETUP();
     if  (funcctx->call_cntr < funcctx->max_calls) {
         src = (inet*)palloc(sizeof(inet));
-        if  (src == NULL)
-                     ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
+        if  (!src) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_set: out of memory")));
+        }
         MemSet(src, 0, sizeof(inet));
         ip_family(src) = PGSQL_AF_INET;
         ip_bits(src) = fctx->masklen;
@@ -221,24 +237,28 @@ ip_set(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(ip_supernet);
 Datum
 ip_supernet(PG_FUNCTION_ARGS) {
-    inet *src;
-    FuncCallContext  *funcctx;
-    MemoryContext oldcontext;
+    inet            *src;
+    FuncCallContext *funcctx;
+    MemoryContext    oldcontext;
 
     if  (SRF_IS_FIRSTCALL()) {
         funcctx = SRF_FIRSTCALL_INIT();
         oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
         if  (PG_ARGISNULL(0)) { SRF_RETURN_DONE(funcctx); }
         src = PG_GETARG_INET_P(0);
-        if  (ip_family(src) == PGSQL_AF_INET6)
-            ereport(ERROR, ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
-                           , errmsg("ip_supernet: address family AF_INET6 not supported yet")
-                   )       );
-        if  (ip_family(src) != PGSQL_AF_INET)
-                               elog(ERROR, "ip_supernet: unknown address family (%d)", ip_family(src));
+        if  (ip_family(src) == PGSQL_AF_INET6) {
+            ereport( ERROR
+                   , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
+                     , errmsg("ip_supernet: address family AF_INET6 not supported yet")
+                   ) );
+        }
+        if  (ip_family(src) != PGSQL_AF_INET) {
+            elog(ERROR, "ip_supernet: unknown address family (%d)", ip_family(src));
+        }
         funcctx->user_fctx = (results*)palloc(sizeof(results));
-        if  (fctx == NULL)
-                ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_supernet: out of memory")));
+        if  (fctx == NULL) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_supernet: out of memory")));
+        }
         MemSet(fctx, 0, sizeof(results));
         funcctx->max_calls = fctx->masklen = ip_bits(src);
         fctx->current = ip_v4addr(src);
@@ -247,11 +267,12 @@ ip_supernet(PG_FUNCTION_ARGS) {
     funcctx = SRF_PERCALL_SETUP();
     if  (funcctx->call_cntr < funcctx->max_calls) {
         src = (inet*)palloc(sizeof(inet));
-        if  (src == NULL)
-                ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_supernet: out of memory")));
+        if  (!src) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ip_supernet: out of memory")));
+        }
         MemSet(src, 0, sizeof(inet));
         ip_family(src) = PGSQL_AF_INET;
-        ip_bits(src) = --(fctx->masklen);
+        ip_bits(src) = (unsigned char)--(fctx->masklen);
 #ifndef NOTYPE
         ip_type(src) = 0;
 #endif
@@ -273,8 +294,9 @@ int4_inet(PG_FUNCTION_ARGS) {
 
     if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
     dst = (inet*)palloc(sizeof(inet));
-    if  (dst == NULL)
-                  ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("int4_inet: out of memory")));
+    if  (dst == NULL) {
+        ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("int4_inet: out of memory")));
+    }
     MemSet(dst, 0, sizeof(inet));
     ip_family(dst) = PGSQL_AF_INET;
     ip_bits(dst) = 32;
@@ -297,8 +319,9 @@ int8_inet(PG_FUNCTION_ARGS) {
 
     if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
     dst = (inet*)palloc(sizeof(inet));
-    if  (dst == NULL)
-                  ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("int8_inet: out of memory")));
+    if  (dst == NULL) {
+        ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("int8_inet: out of memory")));
+    }
     MemSet(dst, 0, sizeof(inet));
     ip_family(dst) = PGSQL_AF_INET;
     ip_bits(dst) = 32;
@@ -321,12 +344,14 @@ inet_int4(PG_FUNCTION_ARGS) {
 
     if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
     src = PG_GETARG_INET_P(0);
-        switch (ip_family(src)) {
-    case PGSQL_AF_INET:  PG_RETURN_INT32(ip_v4addr(src));
+    switch(ip_family(src)) {
+    case PGSQL_AF_INET:
+        PG_RETURN_INT32(ip_v4addr(src));
     case PGSQL_AF_INET6:
-        ereport(ERROR, ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
-                       , errmsg("address AF_INET6 too big for int4")
-               )       );
+        ereport( ERROR
+               , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
+                 , errmsg("address AF_INET6 too big for int4")
+               ) );
     default:
         elog(ERROR, "unknown address family (%d)", ip_family(src));
     }
@@ -340,12 +365,14 @@ inet_int8(PG_FUNCTION_ARGS) {
 
     if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
     src = PG_GETARG_INET_P(0);
-    switch (ip_family(src)) {
-    case PGSQL_AF_INET:  PG_RETURN_INT64(ip_v4addr(src));
+    switch(ip_family(src)) {
+    case PGSQL_AF_INET:
+        PG_RETURN_INT64(ip_v4addr(src));
     case PGSQL_AF_INET6:
-        ereport(ERROR, ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
-                       , errmsg("address AF_INET6 too big for int8")
-               )       );
+        ereport( ERROR
+               , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
+                 , errmsg("address AF_INET6 too big for int8")
+               ) );
     default:
         elog(ERROR, "unknown address family (%d)", ip_family(src));
     }
@@ -367,7 +394,7 @@ qget(struct qstring *qstr) {
         default:  return(ESYM);
 }   }
 
-int
+static int
 qgetnext(struct qstring *qstr) {
     int qchr;
 
@@ -382,7 +409,7 @@ qgetnext(struct qstring *qstr) {
     return(-1);
 }
 
-int
+static int
 qset(struct qstring *qstr, enum qchars qchr) {
     char *ch;
 
@@ -392,7 +419,9 @@ qset(struct qstring *qstr, enum qchars qchr) {
             && *ch != 'f'
             && *ch != 't'
             && (qchr == Q || qchr == T || qchr == F)
-            ) qstr->alength++;
+            ) {
+            qstr->alength++;
+        }
         switch (qchr) {
         case Q: *ch = 'q'; break;
         case T: *ch = 't'; break;
@@ -406,14 +435,16 @@ qset(struct qstring *qstr, enum qchars qchr) {
         return(-1);
 }   }
 
-int 
+static int 
 qsetprev(struct qstring *qstr, enum qchars qchr) {
-    while (qstr->position > 0 && qget(qstr) == SP) qstr->position--;
-    if  (qget(qstr) != SP) return(qset(qstr, qchr));
-      else return(-1);
-}
+    while(qstr->position > 0 && qget(qstr) == SP) qstr->position--;
+    if  (qget(qstr) != SP) {
+        return(qset(qstr, qchr));
+    } else {
+        return(-1);
+}   }
 
-int
+static int
 qputnext(struct qstring *qstr, enum qchars qchr) {
     if  (qstr->position + 1 < qstr->qlength) {
         qstr->position++;
@@ -422,7 +453,7 @@ qputnext(struct qstring *qstr, enum qchars qchr) {
         return(-1);
 }   }
 
-int 
+static int 
 qput(struct qstring *qstr, enum qchars qchr) {
     qstr->position++;
     if  (qstr->position < qstr->qlength) {
@@ -438,14 +469,14 @@ qput(struct qstring *qstr, enum qchars qchr) {
     return(1);
 }
 
-int
+static int
 qreplace(struct qstring *qstr, ipaggr *aa) {
-    int nlength;
+    int     nlength;
     ipaggr *new;
-    void *p;
+    void   *p;
 
     nlength = qstr->qlength + VARSIZE(aa) - VARHDRSZ - 1;
-    new = (ipaggr *)ALLOC(nlength + VARHDRSZ);
+    new = (ipaggr*)ALLOC(nlength + VARHDRSZ);
     if  (new == NULL) {
         ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ipaggr: out of memory")));
         return(-1);
@@ -466,13 +497,14 @@ qreplace(struct qstring *qstr, ipaggr *aa) {
     return(1);
 }
 
-int
+static int
 qdelblank(struct qstring *qstr) {
     ipaggr *new;
-    int32 i, j;
+    int32   i;
+    int32   j;
 	
-    new = (ipaggr *)ALLOC(qstr->alength + VARHDRSZ);
-    if  (new == NULL) {
+    new = (ipaggr*)ALLOC(qstr->alength + VARHDRSZ);
+    if  (!new) {
         ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ipaggr: out of memory")));
         return(-1);
     }
@@ -527,7 +559,7 @@ static const u_int32_t automat[fin+1][5] =
 ,{                   fin,             fin,             fin, fin} /*  fin */
 };
 
-void
+static void
 qstate_init(ipaggr *aa, struct qstate *it) {
     struct qstring *qstr = &it->qstr;
 	
@@ -542,18 +574,18 @@ qstate_init(ipaggr *aa, struct qstate *it) {
         qstr->alength = qstr->qlength;
 }   }
 
-int 
+static int 
 qwalk(struct qstate *it, int action, int32_t s_mask, u_int32_t s_addr) {
+    u_int32_t       control = 0;
     struct qstring *qstr;
-    u_int32_t control = 0;
-    int i;
+    int             i;
 
 retry:
     if  (action == AGGREGATE) {
         qstate_init(NULL, it);
     }
     qstr = &(it->qstr);
-    while (it->state < fin) {
+    while(it->state < fin) {
         if  ((i = qgetnext(qstr)) < 0) {
             ereport( ERROR
                    , ( errcode(ERRCODE_INVALID_TEXT_REPRESENTATION)
@@ -616,21 +648,24 @@ retry:
             }
             break;
         case SET_BLANK:
-            if  (it->mask > s_mask) qset(&it->qstr, SP);
-              else  return(1);
+            if  (it->mask > s_mask) {
+                qset(&it->qstr, SP);
+            } else {
+                return(1);
+            }
 break;
     }   }
     return(0);
 }
 
-ipaggr *
+static ipaggr *
 qgenerate(int level, int mask, u_int32_t addr, enum qchars type) {
     struct qstring qstr;
-    ipaggr *aa;
-    int32 length;
-    int i = level;
-    int pos = 0;
-    enum qchars invtype = T;
+    ipaggr        *aa;
+    int32          length;
+    int            i = level;
+    int            pos = 0;
+    enum qchars    invtype = T;
 
     if  (type == T) invtype = F;
     if  (level > mask) {
@@ -648,7 +683,7 @@ qgenerate(int level, int mask, u_int32_t addr, enum qchars type) {
         return(NULL);
     }
     length = 1 + (mask - level) * 2;
-    aa = (ipaggr *)ALLOC(length + VARHDRSZ);
+    aa = (ipaggr*)ALLOC(length + VARHDRSZ);
 #ifdef VARATT_SIZEP
     VARATT_SIZEP(aa) = length + VARHDRSZ;
 #else
@@ -658,7 +693,6 @@ qgenerate(int level, int mask, u_int32_t addr, enum qchars type) {
         ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ipaggr: out of memory")));
         return(NULL);
     }
-
     qstr.qstring_data = aa;
     qstr.qlength = length;
     qstr.alength = length;
@@ -693,17 +727,19 @@ qgenerate(int level, int mask, u_int32_t addr, enum qchars type) {
     return(aa);
 }
 
-ipaggr *
+static ipaggr *
 qcomp(ipaggr *aa, int mask, u_int32_t addr, enum qchars type) {
-    struct qstate qu;
+    struct qstate  qu;
     struct qstate *qs = &qu;
-    enum qchars invtype = T;
+    enum qchars    invtype = T;
 
     if  (type == T) invtype = F;
-    if  (aa == NULL) {
-        if  (type == T) return(qgenerate(0, mask, addr, T));
-          else return(NULL);
-    }
+    if  (!aa) {
+        if  (type == T) {
+            return(qgenerate(0, mask, addr, T));
+        } else {
+            return(NULL);
+    }   }
     qstate_init(aa, qs);
     if  (qwalk(qs, SEARCH_ADDR, mask, addr) == 1) {
         if  (qs->qc == type) return(aa);
@@ -732,37 +768,39 @@ qcomp(ipaggr *aa, int mask, u_int32_t addr, enum qchars type) {
     return(NULL);
 } 
 
-ipaggr *
+static ipaggr *
 qadd(ipaggr *aa, int mask, u_int32_t addr) {
     return(qcomp(aa, mask, addr, T));
 }
 
-ipaggr *
+static ipaggr *
 qsub(ipaggr *aa, int mask, u_int32_t addr) {
     return(qcomp(aa, mask, addr, F));
 }
 
-ipaggr *
+static ipaggr *
 qintersect(ipaggr *aa, ipaggr *bb) {
-    struct qstate qu,qy;
-    struct qstate *qs = &qu, *qp = &qy;
-    ipaggr *res = NULL, *old = NULL;	
+    struct qstate  qu;
+    struct qstate  qy;
+    struct qstate *qs = &qu;
+    struct qstate *qp = &qy;
+    ipaggr        *res = NULL;	
+    ipaggr        *old = NULL;	
 
-    if  (aa == NULL || bb == NULL) return(NULL);
+    if  (!aa || !bb) return(NULL);
     qstate_init(aa, qs);
     qstate_init(bb, qp);
 	
     if  (qwalk(qp, SEARCH_T, 0, 0) <= 0 || qwalk(qs, SEARCH_T, 0, 0) <= 0) goto end;
-    while (1) {
-        while (qs->mask <= qp->mask &&
-		    qs->addr == SETMASK(qp->addr, qs->mask) && 
-		    qs->qc == T && qp->qc == T) {
-			res = qadd(res, qp->mask, qp->addr);
-			if  (old != res && old != NULL)
-				FREE(old);
-			old = res;
-			if  (qwalk(qp, SEARCH_T, 0, 0) <= 0)
-				goto end;
+    for (; ;) {
+        while(  qs->mask <= qp->mask
+             && qs->addr == SETMASK(qp->addr, qs->mask)
+             && qs->qc == T && qp->qc == T
+             ) {
+            res = qadd(res, qp->mask, qp->addr);
+            if  (old != res && old != NULL) FREE(old);
+            old = res;
+            if  (qwalk(qp, SEARCH_T, 0, 0) <= 0) goto end;
         }
         while (  qp->mask <= qs->mask
               && qp->addr == SETMASK(qs->addr, qp->mask)
@@ -779,13 +817,13 @@ qintersect(ipaggr *aa, ipaggr *bb) {
             if  (qwalk(qs, SEARCH_T, 0, 0) <= 0) goto end;
     }   }
 end:
-    if  (res == NULL) res = qgenerate(0, 0, 0, F);
+    if  (!res) res = qgenerate(0, 0, 0, F);
     return(res);
 }
 
-int
+static int
 qcheck(ipaggr *aa) {
-    struct qstate qu;
+    struct qstate  qu;
     struct qstate *qs = &qu;
 
     qstate_init(aa, qs);
@@ -803,28 +841,30 @@ qcheck(ipaggr *aa) {
 PG_FUNCTION_INFO_V1(ipaggr_in);
 Datum
 ipaggr_in(PG_FUNCTION_ARGS) {
-    char *cstr = (char *)PG_GETARG_CSTRING(0);
-    char *p = cstr;
-    int32 len = 0;
+    char   *cstr = (char*)PG_GETARG_CSTRING(0);
+    char   *p = cstr;
+    int32   len = 0;
     ipaggr *result;
 
     len = strlen(cstr);
-    while (*p) {
-		switch (*p) {
-		case 'q':
-		case 't':
-		case 'f':
-			break;
-		default:	
-			ereport(ERROR,
-			    (errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
-			    errmsg("invalid input syntax for ipaggr")));
-		}
-		p++;
+    while(*p) {
+        switch (*p) {
+        case 'q':
+        case 't':
+        case 'f':
+            break;
+        default:	
+            ereport( ERROR
+                   , ( errcode(ERRCODE_INVALID_TEXT_REPRESENTATION)
+                     , errmsg("invalid input syntax for ipaggr")
+                   ) );
+        }
+        p++;
     }
-    result = (ipaggr *)ALLOC(len + VARHDRSZ);
-    if  (result == NULL)
+    result = (ipaggr*)ALLOC(len + VARHDRSZ);
+    if  (!result) {
         ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ipaggr: out of memory")));
+    }
     memcpy(VARDATA(result), cstr, len);
 #ifdef VARATT_SIZEP
     VARATT_SIZEP(result) = len + VARHDRSZ;
@@ -839,14 +879,15 @@ PG_FUNCTION_INFO_V1(ipaggr_out);
 Datum
 ipaggr_out(PG_FUNCTION_ARGS) {
     ipaggr *aggr;
-    char *cstr;
-    int32 len;
+    char   *cstr;
+    int32   len;
 
-    aggr = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
+    aggr = (ipaggr*)PG_GETARG_VARLENA_P(0);
     len = VARSIZE(aggr) - VARHDRSZ;
-    cstr = (char *)ALLOC(len + 1);
-    if  (cstr == NULL)
+    cstr = (char*)ALLOC(len + 1);
+    if  (!cstr) {
         ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("ipaggr: out of memory")));
+    }
     memcpy(cstr, VARDATA(aggr), len);
     cstr[len] = '\0';
     PG_RETURN_CSTRING(cstr);
@@ -855,46 +896,51 @@ ipaggr_out(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(check_net);
 Datum
 check_net(PG_FUNCTION_ARGS) {
-    ipaggr *aggr;
-    inet *net;
-    u_int32_t s_addr;
-    int s_mask;
-    struct qstate qu;
+    ipaggr        *aggr;
+    inet          *net;
+    u_int32_t      s_addr;
+    int            s_mask;
+    struct qstate  qu;
     struct qstate *qs = &qu;
 
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
+    aggr = (ipaggr*)PG_GETARG_VARLENA_P(0);
     qstate_init(aggr, qs);
 
     net = PG_GETARG_INET_P(1);
-    if  (ip_family(net) == PGSQL_AF_INET6)
+    if  (ip_family(net) == PGSQL_AF_INET6) {
         ereport( ERROR
                , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
                  , errmsg("add_net: address family AF_INET6 not supported yet")
                ) );
-	s_mask = ip_bits(net);
-	s_addr = ip_v4addr(net);
+    }
+    s_mask = ip_bits(net);
+    s_addr = ip_v4addr(net);
 
-	if  (qwalk(qs, SEARCH_ADDR, s_mask, s_addr) > 0 && qs->qc == T) PG_RETURN_BOOL(true);
-	  else PG_RETURN_BOOL(false);
-}
+    if  (qwalk(qs, SEARCH_ADDR, s_mask, s_addr) > 0 && qs->qc == T) {
+        PG_RETURN_BOOL(true);
+    } else {
+        PG_RETURN_BOOL(false);
+}   }
 
 PG_FUNCTION_INFO_V1(add_net);
 Datum
 add_net(PG_FUNCTION_ARGS) {
-    ipaggr *aggr, *newaggr;
-    inet *net;
-    u_long s_addr;
-    int s_mask;
+    ipaggr *newaggr;
+    ipaggr *aggr;
+    inet   *net;
+    u_long  s_addr;
+    int     s_mask;
 
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
+    aggr = (ipaggr*)PG_GETARG_VARLENA_P(0);
     net = PG_GETARG_INET_P(1);
-    if  (ip_family(net) == PGSQL_AF_INET6)
+    if  (ip_family(net) == PGSQL_AF_INET6) {
         ereport( ERROR
                , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
                  , errmsg("add_net: address family AF_INET6 not supported yet")
                ) );
+    }
     s_mask = ip_bits(net);
     s_addr = ip_v4addr(net);
     newaggr = qadd(aggr, s_mask, s_addr);
@@ -904,19 +950,21 @@ add_net(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(sub_net);
 Datum
 sub_net(PG_FUNCTION_ARGS) {
-    ipaggr *aggr, *newaggr;
-    inet *net;
+    ipaggr   *newaggr;
+    ipaggr   *aggr;
+    inet     *net;
     u_int32_t s_addr;
-    int s_mask;
+    int       s_mask;
 	
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
+    aggr = (ipaggr*)PG_GETARG_VARLENA_P(0);
     net = PG_GETARG_INET_P(1);
-    if  (ip_family(net) == PGSQL_AF_INET6)
+    if  (ip_family(net) == PGSQL_AF_INET6) {
         ereport( ERROR
                , ( errcode(ERRCODE_FEATURE_NOT_SUPPORTED)
                  , errmsg("add_net: address family AF_INET6 not supported yet")
                ) );
+    }
     s_mask = ip_bits(net);
     s_addr = ip_v4addr(net);
     newaggr = qsub(aggr, s_mask, s_addr);
@@ -926,13 +974,16 @@ sub_net(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(add_ipaggr);
 Datum
 add_ipaggr(PG_FUNCTION_ARGS) {
-    ipaggr *aggr1, *aggr2, *new, *old;
-    struct qstate qu;
+    ipaggr        *aggr1;
+    ipaggr        *aggr2;
+    ipaggr        *new;
+    ipaggr        *old;
+    struct qstate  qu;
     struct qstate *qs = &qu;
 
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr1 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
-    aggr2 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(1));
+    aggr1 = (ipaggr*)PG_GETARG_VARLENA_P(0);
+    aggr2 = (ipaggr*)PG_GETARG_VARLENA_P(1);
     qstate_init(aggr2, qs);
     new = aggr1;
     old = aggr1;
@@ -947,13 +998,16 @@ add_ipaggr(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(sub_ipaggr);
 Datum
 sub_ipaggr(PG_FUNCTION_ARGS) {
-    ipaggr *aggr1, *aggr2, *new, *old;
-    struct qstate qu;
+    ipaggr        *aggr1;
+    ipaggr        *aggr2;
+    ipaggr        *new;
+    ipaggr        *old;
+    struct qstate  qu;
     struct qstate *qs = &qu;
 
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr1 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
-    aggr2 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(1));
+    aggr1 = (ipaggr*)PG_GETARG_VARLENA_P(0);
+    aggr2 = (ipaggr*)PG_GETARG_VARLENA_P(1);
     qstate_init(aggr2, qs);
     new = aggr1;
     old = aggr1;
@@ -968,11 +1022,13 @@ sub_ipaggr(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(inter_ipaggr);
 Datum
 inter_ipaggr(PG_FUNCTION_ARGS) {
-    ipaggr *aggr1, *aggr2, *new;
+    ipaggr *aggr1;
+    ipaggr *aggr2;
+    ipaggr *new;
 
     if  (PG_ARGISNULL(0) || PG_ARGISNULL(1)) PG_RETURN_NULL();
-    aggr1 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(0));
-    aggr2 = (ipaggr *)heap_tuple_untoast_attr((varattrib *)PG_GETARG_POINTER(1));
+    aggr1 = (ipaggr*)PG_GETARG_VARLENA_P(0);
+    aggr2 = (ipaggr*)PG_GETARG_VARLENA_P(1);
     new = qintersect(aggr1, aggr2);
     PG_RETURN_POINTER(new);
 }
@@ -980,33 +1036,34 @@ inter_ipaggr(PG_FUNCTION_ARGS) {
 PG_FUNCTION_INFO_V1(show_net);
 Datum
 show_net(PG_FUNCTION_ARGS) {
-    inet *net;
-    ipaggr *aggr;
-    struct qstate *qs;
-    FuncCallContext  *funcctx;
+    inet            *net;
+    ipaggr          *aggr;
+    struct qstate   *qs;
+    FuncCallContext *funcctx;
 
     if  (SRF_IS_FIRSTCALL()) {
         MemoryContext oldcontext;
         funcctx = SRF_FIRSTCALL_INIT();
         oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
         if  (PG_ARGISNULL(0)) SRF_RETURN_DONE(funcctx);
-        aggr = (ipaggr *)PG_GETARG_POINTER(0);
-        aggr = (ipaggr *)heap_tuple_untoast_attr((varattrib *)aggr);
+        aggr = (ipaggr*)PG_GETARG_VARLENA_P(0);
         qs = (struct qstate *)ALLOC(sizeof(struct qstate));
-        if  (qs == NULL)
+        if  (!qs) {
             ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("show_net: out of memory")));
+        }
         MemSet(qs, 0, sizeof(struct qstate));
         qstate_init(aggr, qs);
-        funcctx->user_fctx = (void *)qs;
+        funcctx->user_fctx = (void*)qs;
         MemoryContextSwitchTo(oldcontext);
     }
     funcctx = SRF_PERCALL_SETUP();
     qs = (struct qstate *)funcctx->user_fctx;
     aggr = qs->qstr.qstring_data;
     if  (qwalk(qs, SEARCH_T, 0, 0) > 0) {
-        net = (inet *)ALLOC(sizeof(inet));
-        if  (net == NULL)
+        net = (inet*)ALLOC(sizeof(inet));
+        if  (!net) {
             ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("show_net: out of memory")));
+        }
         MemSet(net, 0, sizeof(inet));
         ip_family(net) = PGSQL_AF_INET;
         ip_bits(net) = qs->mask;
@@ -1019,7 +1076,7 @@ show_net(PG_FUNCTION_ARGS) {
 #else
         SET_VARSIZE(net, sizeof(inet_struct) + VARHDRSZ);
 #endif
-        funcctx->user_fctx = (void *)qs;
+        funcctx->user_fctx = (void*)qs;
         SRF_RETURN_NEXT(funcctx, InetPGetDatum(net));
     } else {
         SRF_RETURN_DONE(funcctx);
