@@ -1,6 +1,7 @@
 /*-
- * Copyright (C) @BABOLO  2002 Nov 03
+ * Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/
  * Portions Copyright (c) 2000, PostgreSQL Global Development Group
+ * PKG = babolo_pglib
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -25,12 +26,9 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-static const char copyright[] = "\
-@(#)Copyright (C) @BABOLO  2002 Nov 03\n\
-@(#)All rights reserved.\n";
-static const char rcsid[] = "$Id: pglib_text.c,v 1.5 2016/07/12 22:02:34 babolo Exp $";
-#endif /* not lint */
+#ident "@(#) Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) Portions Copyright (c) 2000, PostgreSQL Global Development Group"
+#ident "@(#) $Id: pglib_text.c,v 1.10 2025/11/12 02:39:09 babolo Exp $"
 
 #include <sys/param.h>
 #include <sys/types.h>
@@ -117,30 +115,32 @@ Datum quote_html(PG_FUNCTION_ARGS) {
     int rq, len;
 
     cp2 = VARDATA(t);
-    cp1 = VARDATA(t) + VARSIZE(t) - VARHDRSZ;
+    cp1 = cp2 + VARSIZE(t) - VARHDRSZ;
     for (rq = 0; cp2 < cp1; cp2++) rq += subst_in[*(u_char*)cp2];
 
     if  (rq) {
         len = VARSIZE(t) - VARHDRSZ;
-        result = (text *) palloc(len + rq + VARHDRSZ);
-        if  (result == NULL)
-                   ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        result = (text*)palloc(len + rq + VARHDRSZ);
+        if  (!result) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        }
         cp2 = VARDATA(result);
         for (cp1 = VARDATA(t); len-- > 0; cp1++) {
             cp0 = subst_out[*(u_char*)cp1];
-            do  {
+            do {
                 *cp2++ = *cp0++;
             } while (*cp0);
 	}
 #ifdef VARATT_SIZEP
-        VARATT_SIZEP(result) = cp2 - ((char *) result);
+        VARATT_SIZEP(result) = cp2 - ((char*)result);
 #else
-        SET_VARSIZE(result, cp2 - ((char *) result));
+        SET_VARSIZE(result, cp2 - ((char*)result));
 #endif
     } else {
-        result = (text *) palloc(VARSIZE(t));
-        if  (result == NULL)
-                   ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        result = (text*)palloc(VARSIZE(t));
+        if  (!result) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        }
         memcpy(result, t, VARSIZE(t));
     }
 
@@ -149,7 +149,6 @@ Datum quote_html(PG_FUNCTION_ARGS) {
 }
 
 #define C_bq     0x2000 /*   \                                                                 */
-#define C_aq     0x1000 /*                                                      */
 #define C_dq      0x400 /*                                       */
 #define C_sq      0x200 /*                                     */
 #define C_pl      0x100 /*                                            */
@@ -213,8 +212,9 @@ Datum quote_sh(PG_FUNCTION_ARGS) {
     len = VARSIZE(t) - VARHDRSZ;
     for (state = c0; len-- >= 0; cp1++) {
         control = automa[state][(len >= 0) ? subst_ih[(u_char)*cp1] : ze];
-        ELOG((DEBUG, "quote_sh:%d: %08X %02X"
-            , state, control, (len >= 0) ? subst_ih[(u_char)*cp1] : ze
+        ELOG((DEBUG
+             , "quote_sh:%d: %08X %02X"
+             , state, control, (len >= 0) ? subst_ih[(u_char)*cp1] : ze
             ));
         if  (control & C_sq) {
             rp += (rq[5] || (rq[1] + rq[2] + rq[4]) > 1) ? 2 : (rq[1] + rq[2] + rq[4]);
@@ -230,22 +230,26 @@ Datum quote_sh(PG_FUNCTION_ARGS) {
     ELOG((DEBUG, "quote_sh: %d", rp));
     if  (rp) {
         len = VARSIZE(t) - VARHDRSZ;
-        result = (text *) palloc(len + rp + VARHDRSZ);
-        if  (result == NULL)
-                   ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        result = (text*)palloc(len + rp + VARHDRSZ);
+        if  (result == NULL) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        }
         cp0 = cp1 = VARDATA(t);
         cp2 = VARDATA(result);
         for (state = c0; len-- >= 0; cp1++) {
             control = automa[state][(len >= 0) ? subst_ih[(u_char)*cp1] : ze];
-            ELOG((DEBUG, "quote_sh:%d: %08X %02X"
-                , state, control, (len >= 0) ? subst_ih[(u_char)*cp1] : ze
+            ELOG(( DEBUG
+                 , "quote_sh:%d: %08X %02X"
+                 , state, control, (len >= 0) ? subst_ih[(u_char)*cp1] : ze
                 ));
             if  (control & C_sq) {
                 if  (rq[5] || (rq[1] + rq[2] + rq[4]) > 1) {
                     *cp2++ = '\'';
                     for (; cp0 < cp1; ++cp0) *cp2++ = *cp0;
                     *cp2++ = '\'';
-                } else control |= C_bq;
+                } else {
+                    control |= C_bq;
+                }
                 bzero(rq, sizeof(rq));
             }
             if  (control & C_dq) {
@@ -256,7 +260,9 @@ Datum quote_sh(PG_FUNCTION_ARGS) {
                         *cp2++ = *cp0;
                     }
                     *cp2++ = '\"';
-                } else control |= C_bq;
+                } else {
+                    control |= C_bq;
+                }
                 bzero(rq, sizeof(rq));
             }
             if  (control & C_bq) {
@@ -268,14 +274,15 @@ Datum quote_sh(PG_FUNCTION_ARGS) {
             state = control & C_state;
 	}
 #ifdef VARATT_SIZEP
-        VARATT_SIZEP(result) = cp2 - ((char *) result);
+        VARATT_SIZEP(result) = cp2 - ((char*)result);
 #else
-        SET_VARSIZE(result, cp2 - ((char *) result));
+        SET_VARSIZE(result, cp2 - ((char*)result));
 #endif
     } else {
-        result = (text *) palloc(VARSIZE(t));
-        if  (result == NULL)
-                   ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        result = (text*)palloc(VARSIZE(t));
+        if  (!result) {
+            ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_sh: out of memory")));
+        }
         memcpy(result, t, VARSIZE(t));
     }
 
@@ -326,19 +333,19 @@ Datum quote_copy(PG_FUNCTION_ARGS) {
     } else {
         t = PG_GETARG_TEXT_P(0);
         cp1 = VARDATA(t);
-        len = VARSIZE(t);
+        len = VARSIZE(t) - VARHDRSZ;
         for (rq = 0, i = 0; i < len; i++) rq += (copy_subst[cp1[i] & 0xFF] < ' ') ? 0 : 1;
 
         if  (rq) {
-            len -= VARHDRSZ;
             result = (text*)palloc(len + rq + VARHDRSZ);
+            if  (!result) {
+                ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_copy: out of memory")));
+            }
 #ifdef VARATT_SIZEP
             VARATT_SIZEP(result) = len + rq + VARHDRSZ;
 #else
             SET_VARSIZE(result, len + rq + VARHDRSZ);
 #endif
-            if  (!result)
-                ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_copy: out of memory")));
             cp2 = VARDATA(result);
             for (i = 0; i < len; i++) {
                 if  (copy_subst[cp1[i] & 0xFF] < ' ') {
@@ -348,12 +355,159 @@ Datum quote_copy(PG_FUNCTION_ARGS) {
                     *(cp2++) = copy_subst[cp1[i] & 0xFF];
             }   }
         } else {
-            result = (text*)palloc(len);
-            if  (!result)
+            result = (text*)palloc(len + VARHDRSZ);
+            if  (!result) {
                 ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("quote_copy: out of memory")));
-            memcpy(result, t, len);
+            }
+            memcpy(result, t, len + VARHDRSZ);
         }
         PG_FREE_IF_COPY(t, 0);
     }
     PG_RETURN_TEXT_P(result);
 }
+
+static const u_char koiup[] =
+{    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20,    0,     0,    0,    0,    0
+
+,    0,    0,    0,    2,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+, 0x10,    0,    0,    0,     0,    0,    0,    0,     0,    0, 0x10,    0,  0x10, 0x10, 0x10, 0x10
+,    0, 0x10, 0x10, 0x10,  0x10, 0x10, 0x10, 0x10,  0x10,    0,    0,    0,  0x10, 0x10, 0x10,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+};
+
+static const u_char koidn[] =
+{    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+
+,    0, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+
+, 0x10,    2,    0,    0,     0,    0,    0,    0,     0,    0, 0x10,    0,  0x10, 0x10, 0x10, 0x10
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0, 0x10, 0x10, 0x10,  0x10, 0x10, 0x10, 0x10,  0x10,    0,    0,    0,  0x10, 0x10, 0x10,    0
+
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+,    0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0,     0,    0,    0,    0
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+, 0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20,  0x20, 0x20, 0x20, 0x20
+};
+
+PG_FUNCTION_INFO_V1(koiupper);
+Datum koiupper(PG_FUNCTION_ARGS) {
+    text    *result;
+    u_char  *c1;
+    u_char  *c2;
+    text    *t;
+    int      p;
+
+    if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
+    t = PG_GETARG_TEXT_P(0);
+    if  (!(result = (text*)palloc(VARSIZE(t) + VARHDRSZ))) {
+        ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("text_bytea: out of memory")));
+    }
+    c1 = (u_char*)VARDATA(t);
+#ifdef VARATT_SIZEP
+        VARATT_SIZEP(result) = VARSIZE(t);
+#else
+        SET_VARSIZE(result, VARSIZE(t));
+#endif
+    c2 = (u_char*)VARDATA(result);
+    for (p = 0; p < ((int)VARSIZE(t) - VARHDRSZ); ++p) {
+        c2[p] = c1[p] ^ koiup[c1[p] & 0xFF];
+    }
+    PG_FREE_IF_COPY(t, 0);
+    PG_RETURN_TEXT_P(result);
+}
+
+PG_FUNCTION_INFO_V1(koilower);
+Datum koilower(PG_FUNCTION_ARGS) {
+    text    *result;
+    u_char  *c1;
+    u_char  *c2;
+    text    *t;
+    int    p;
+
+    if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
+    t = PG_GETARG_TEXT_P(0);
+    if  (!(result = (text*)palloc(VARSIZE(t) + VARHDRSZ))) {
+        ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("text_bytea: out of memory")));
+    }
+    c1 = (u_char*)VARDATA(t);
+#ifdef VARATT_SIZEP
+        VARATT_SIZEP(result) = VARSIZE(t);
+#else
+        SET_VARSIZE(result, VARSIZE(t));
+#endif
+    c2 = (u_char*)VARDATA(result);
+    for (p = 0; p < ((int)VARSIZE(t) - VARHDRSZ); ++p) {
+        c2[p] = c1[p] ^ koidn[c1[p] & 0xFF];
+    }
+    PG_FREE_IF_COPY(t, 0);
+    PG_RETURN_TEXT_P(result);
+}
+
+static const u_char koisc[] =
+{    0,  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' '
+,  ' ',  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' ',   ' ',  ' ',  ' ',  ' '
+,  ' ',  '.',  '.',  '#',   '.',  '%',  '.',  '&',   '.',  '.',  '.',  '.',   ',',  '-',  '.',  '/'
+,  '0',  '1',  '2',  '3',   '4',  '5',  '6',  '7',   '8',  '9',  ':',  ';',   '<',  '=',  '>',  '.'
+
+,  '@',  '',  'b',  '',   'd',  '',  'f',  'g',   '',  'i',  'j',  '',   '1',  'm',  'n',  '0'
+,  '',  'q',  'r',  's',   '',  '',  'v',  'w',   '',  '',  'z',  '.',   '.',  '.',  '.',  '_'
+,  '.',  '',  'b',  '',   'd',  '',  'f',  'g',   '',  'i',  'j',  '',   '1',  'm',  'n',  '0'
+,  '',  'q',  'r',  's',   '',  '',  'v',  'w',   '',  '',  'z',  '.',   '.',  '.',  '~',  '.'
+
+,  '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.'
+,  '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.',   '.',  '.',  ' ',  '.',   '0',  '2',  '.',  '.'
+,  '.',  '.',  '.',  '',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.'
+,  '.',  '.',  '.',  '',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  '.',   '.',  '.',  '.',  ''
+
+,  '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '0'
+,  '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
+,  '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '0'
+,  '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  '',   '',  '',  '',  ''
+};
+
+PG_FUNCTION_INFO_V1(koisearch);
+Datum koisearch(PG_FUNCTION_ARGS) {
+    text    *result;
+    u_char  *c1;
+    u_char  *c2;
+    text    *t;
+    int    p;
+
+    if  (PG_ARGISNULL(0)) PG_RETURN_NULL();
+    t = PG_GETARG_TEXT_P(0);
+    if  (!(result = (text*)palloc(VARSIZE(t) + VARHDRSZ))) {
+        ereport(ERROR, (errcode(ERRCODE_OUT_OF_MEMORY), errmsg("text_bytea: out of memory")));
+    }
+    c1 = (u_char*)VARDATA(t);
+#ifdef VARATT_SIZEP
+        VARATT_SIZEP(result) = VARSIZE(t);
+#else
+        SET_VARSIZE(result, VARSIZE(t));
+#endif
+    c2 = (u_char*)VARDATA(result);
+    for (p = 0; p < ((int)VARSIZE(t) - VARHDRSZ); ++p) {
+        c2[p] = koisc[c1[p]];
+    }
+    PG_FREE_IF_COPY(t, 0);
+    PG_RETURN_TEXT_P(result);
+}
